@@ -7,6 +7,7 @@ from functools import partial
 from sql import Table, Null
 from sql.functions import Overlay, Position
 from sql.operators import Concat
+from sql.conditionals import Case
 
 from trytond.model import Workflow, ModelView, ModelSQL, fields
 from trytond.modules.company import CompanyReport
@@ -82,12 +83,21 @@ class Sale(Workflow, ModelSQL, ModelView, TaxableMixin):
             'readonly': Eval('state') != 'draft',
             'required': ~Eval('state').in_(['draft', 'quotation', 'cancel']),
             }, depends=['state', 'party'])
+    shipment_party = fields.Many2One('party.party', 'Shipment Party',
+        states={
+            'readonly': (Eval('state') != 'draft'),
+            },
+        depends=['state'])
     shipment_address = fields.Many2One('party.address', 'Shipment Address',
-        domain=[('party', '=', Eval('party'))], states={
+        domain=[
+            ('party', '=', If(Bool(Eval('shipment_party')),
+                    Eval('shipment_party'), Eval('party'))),
+            ],
+        states={
             'readonly': Eval('state') != 'draft',
             'required': ~Eval('state').in_(['draft', 'quotation', 'cancel']),
             },
-        depends=['party', 'state'])
+        depends=['party', 'shipment_party', 'state'])
     warehouse = fields.Many2One('stock.location', 'Warehouse',
         domain=[('type', '=', 'warehouse')], states={
             'readonly': Eval('state') != 'draft',
@@ -169,8 +179,7 @@ class Sale(Workflow, ModelSQL, ModelView, TaxableMixin):
     shipment_returns = fields.Function(
         fields.One2Many('stock.shipment.out.return', None, 'Shipment Returns'),
         'get_shipment_returns', searcher='search_shipment_returns')
-    moves = fields.Function(fields.One2Many('stock.move', None, 'Moves'),
-        'get_moves')
+    moves = fields.One2Many('stock.move', 'sale', 'Moves', readonly=True)
     origin = fields.Reference('Origin', selection='get_origin', select=True,
         states={
             'readonly': Eval('state') != 'draft',
@@ -384,17 +393,27 @@ class Sale(Workflow, ModelSQL, ModelView, TaxableMixin):
     def default_shipment_state():
         return 'none'
 
-    @fields.depends('party', 'payment_term')
+    @fields.depends('party', 'shipment_party', 'payment_term')
     def on_change_party(self):
         self.invoice_address = None
-        self.shipment_address = None
+        if not self.shipment_party:
+            self.shipment_address = None
         self.payment_term = self.default_payment_term()
         if self.party:
             self.invoice_address = self.party.address_get(type='invoice')
-            self.shipment_address = self.party.address_get(type='delivery')
+            if not self.shipment_party:
+                self.shipment_address = self.party.address_get(type='delivery')
             if self.party.customer_payment_term:
                 self.payment_term = self.party.customer_payment_term
 
+    @fields.depends('party', 'shipment_party')
+    def on_change_shipment_party(self):
+        if self.shipment_party:
+            self.shipment_address = self.shipment_party.address_get(
+                type='delivery')
+        elif self.party:
+            self.shipment_address = self.party.address_get(type='delivery')
+
     @fields.depends('currency')
     def on_change_with_currency_digits(self, name=None):
         if self.currency:
@@ -510,7 +529,7 @@ class Sale(Workflow, ModelSQL, ModelView, TaxableMixin):
 
     @classmethod
     def search_invoices(cls, name, clause):
-        return [('lines.invoice_lines.invoice.id',) + tuple(clause[1:])]
+        return [('lines.invoice_lines.invoice',) + tuple(clause[1:])]
 
     def get_invoice_state(self):
         '''
@@ -558,7 +577,7 @@ class Sale(Workflow, ModelSQL, ModelView, TaxableMixin):
         Search on shipments or returns
         '''
         def method(self, name, clause):
-            return [('lines.moves.shipment.id',) + tuple(clause[1:])
+            return [('lines.moves.shipment',) + tuple(clause[1:])
                 + (model_name,)]
         return classmethod(method)
 
@@ -566,9 +585,6 @@ class Sale(Workflow, ModelSQL, ModelView, TaxableMixin):
     search_shipment_returns = search_shipments_returns(
         'stock.shipment.out.return')
 
-    def get_moves(self, name):
-        return [m.id for l in self.lines for m in l.moves]
-
     def get_shipment_state(self):
         '''
         Return the shipment state for the sale.
@@ -641,7 +657,7 @@ class Sale(Workflow, ModelSQL, ModelView, TaxableMixin):
 
     @classmethod
     def view_attributes(cls):
-        return [('//field[@name="comment"]', 'spell', Eval('party_lang'))]
+        return [('/form//field[@name="comment"]', 'spell', Eval('party_lang'))]
 
     @classmethod
     def copy(cls, sales, default=None):
@@ -652,6 +668,7 @@ class Sale(Workflow, ModelSQL, ModelView, TaxableMixin):
         default['reference'] = None
         default['invoice_state'] = 'none'
         default['invoices_ignored'] = None
+        default['moves'] = None
         default['shipment_state'] = 'none'
         default.setdefault('sale_date', None)
         return super(Sale, cls).copy(sales, default=default)
@@ -764,7 +781,8 @@ class Sale(Workflow, ModelSQL, ModelView, TaxableMixin):
         invoice = self._get_invoice_sale(invoice_type)
         invoice.lines = ((list(invoice.lines)
                 if hasattr(invoice, 'lines') else [])
-            + list(chain.from_iterable(invoice_lines.itervalues())))
+            + list(chain.from_iterable(invoice_lines[l.id] for l in self.lines
+                    if l.id in invoice_lines)))
         invoice.save()
 
         Invoice.update_taxes([invoice])
@@ -801,7 +819,8 @@ class Sale(Workflow, ModelSQL, ModelView, TaxableMixin):
 
     def _get_shipment_sale(self, Shipment, key):
         values = {
-            'customer': self.party.id,
+            'customer': (self.shipment_party.id if self.shipment_party
+                else self.party.id),
             'delivery_address': self.shipment_address.id,
             'company': self.company.id,
             }
@@ -1042,11 +1061,9 @@ class SaleLine(ModelSQL, ModelView):
             'To Location'), 'get_to_location')
     delivery_date = fields.Function(fields.Date('Delivery Date',
             states={
-                'invisible': ((Eval('type') != 'line')
-                    | (If(Bool(Eval('quantity')), Eval('quantity', 0), 0)
-                        <= 0)),
+                'invisible': Eval('type') != 'line',
                 },
-            depends=['type', 'quantity']),
+            depends=['type']),
         'on_change_with_delivery_date')
 
     @classmethod
@@ -1084,7 +1101,7 @@ class SaleLine(ModelSQL, ModelView):
     @staticmethod
     def order_sequence(tables):
         table, _ = tables[None]
-        return [table.sequence == Null, table.sequence]
+        return [Case((table.sequence == Null, 0), else_=1), table.sequence]
 
     @staticmethod
     def default_type():
@@ -1278,8 +1295,11 @@ class SaleLine(ModelSQL, ModelView):
             if self.warehouse:
                 return self.warehouse.input_location.id
 
-    @fields.depends('product', 'quantity', '_parent_sale.sale_date')
+    @fields.depends('product', 'quantity', 'moves', '_parent_sale.sale_date')
     def on_change_with_delivery_date(self, name=None):
+        if self.moves:
+            return min(m.effective_date if m.effective_date else m.planned_date
+                for m in self.moves)
         if self.product and self.quantity > 0:
             date = self.sale.sale_date if self.sale else None
             return self.product.compute_delivery_date(date=date)
@@ -1377,7 +1397,8 @@ class SaleLine(ModelSQL, ModelView):
 
     @classmethod
     def view_attributes(cls):
-        return [('//field[@name="note"]|//field[@name="description"]',
+        return [
+            ('/form//field[@name="note"]|/form//field[@name="description"]',
                 'spell', Eval('_parent_sale', {}).get('party_lang'))]
 
     @classmethod
@@ -1505,13 +1526,7 @@ class OpenCustomer(Wizard):
         customer_ids = [line[0] for line in Transaction().cursor.fetchall()]
         action['pyson_domain'] = PYSONEncoder().encode(
             [('id', 'in', customer_ids)])
-
-        model_data, = ModelData.search([
-                ('fs_id', '=', 'act_open_customer'),
-                ('module', '=', 'sale'),
-                ], limit=1)
-        wizard = Wizard(model_data.db_id)
-
+        wizard = Wizard(ModelData.get_id('sale', 'act_open_customer'))
         action['name'] = wizard.name
         return action, {}
 
