@@ -1,5 +1,6 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
+from sql import Null
 from sql.operators import Concat
 
 from trytond.model import Workflow, ModelView, fields
@@ -18,9 +19,9 @@ class ShipmentOut:
     def __setup__(cls):
         super(ShipmentOut, cls).__setup__()
         cls._error_messages.update({
-                'reset_move': 'You cannot reset to draft a move generated '
-                    'by a sale.',
-            })
+                'reset_move': ('You cannot reset to draft a move generated '
+                    'by a sale.'),
+                })
 
     @classmethod
     def write(cls, *args):
@@ -68,9 +69,9 @@ class ShipmentOutReturn:
     def __setup__(cls):
         super(ShipmentOutReturn, cls).__setup__()
         cls._error_messages.update({
-                'reset_move': 'You cannot reset to draft a move generated '
-                    'by a sale.',
-            })
+                'reset_move': ('You cannot reset to draft a move generated '
+                    'by a sale.'),
+                })
 
     @classmethod
     def write(cls, *args):
@@ -140,7 +141,7 @@ class Move:
             cursor.execute(*sql_table.update(
                     columns=[sql_table.origin],
                     values=[Concat('sale.line,', sql_table.sale_line)],
-                    where=sql_table.sale_line != None))
+                    where=sql_table.sale_line != Null))
             table.drop_column('sale_line')
 
     @classmethod
